@@ -1,4 +1,3 @@
-/* $XFree86: xc/programs/twm/menus.c,v 1.18tsi Exp $ */
 /*****************************************************************************/
 /*
 
@@ -52,8 +51,6 @@ in this Software without prior written authorization from The Open Group.
 
 
 /***********************************************************************
- *
- * $Xorg: menus.c,v 1.6 2001/02/09 02:05:36 xorgcvs Exp $
  *
  * twm menu code
  *
@@ -61,6 +58,10 @@ in this Software without prior written authorization from The Open Group.
  *
  ***********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <stdio.h>
 #include <X11/Xos.h>
 #include "twm.h"
@@ -97,7 +98,7 @@ int ConstMoveXL;
 int ConstMoveXR;
 int ConstMoveYT;
 int ConstMoveYB;
- 
+
 /* Globals used to keep track of whether the mouse has moved during
    a resize function. */
 int ResizeOrigX;
@@ -113,6 +114,16 @@ static Cursor LastCursor;
 static Bool belongs_to_twm_window ( TwmWindow *t, Window w );
 static void Identify ( TwmWindow *t );
 static void send_clientmessage ( Window w, Atom a, Time timestamp );
+static void BumpWindowColormap ( TwmWindow *tmp, int inc );
+static int  DeferExecution ( int context, int func, Cursor cursor );
+static Bool NeedToDefer ( MenuRoot *root );
+static void DestroyMenu ( MenuRoot *menu );
+static void MakeMenu ( MenuRoot *mr );
+static void Execute ( const char *s );
+static void HideIconManager ( void );
+static void WarpAlongRing ( XButtonEvent *ev, Bool forward );
+static int  WarpThere ( TwmWindow * t );
+static void WarpToWindow ( TwmWindow *t );
 
 #define SHADOWWIDTH 5			/* in pixels */
 
@@ -123,7 +134,7 @@ static void send_clientmessage ( Window w, Atom a, Time timestamp );
  * initialize menu roots
  */
 void
-InitMenus()
+InitMenus(void)
 {
     int i, j, k;
     FuncKey *key, *tmp;
@@ -146,7 +157,7 @@ InitMenus()
 	    free(key->name);
 	    tmp = key;
 	    key = key->next;
-	    free((char *) tmp);
+	    free(tmp);
 	}
 	Scr->FuncKeyRoot.next = NULL;
     }
@@ -165,7 +176,7 @@ InitMenus()
  *  \param win_name the window name (if any)
  *  \param action   the action string associated with the function (if any)
  */
-Bool AddFuncKey (char *name, int cont, int mods, int func, char *win_name, 
+Bool AddFuncKey (char *name, int cont, int mods, int func, char *win_name,
                  char *action)
 {
     FuncKey *tmp;
@@ -193,7 +204,7 @@ Bool AddFuncKey (char *name, int cont, int mods, int func, char *win_name,
 
     if (tmp == NULL)
     {
-	tmp = (FuncKey *) malloc(sizeof(FuncKey));
+	tmp = malloc(sizeof(FuncKey));
 	tmp->next = Scr->FuncKeyRoot.next;
 	Scr->FuncKeyRoot.next = tmp;
     }
@@ -212,10 +223,10 @@ Bool AddFuncKey (char *name, int cont, int mods, int func, char *win_name,
 
 
 
-int CreateTitleButton (char *name, int func, char *action, MenuRoot *menuroot, 
-                       Bool rightside, Bool append)
+int CreateTitleButton (const char *name, int func, const char *action,
+                       MenuRoot *menuroot, Bool rightside, Bool append)
 {
-    TitleButton *tb = (TitleButton *) malloc (sizeof(TitleButton));
+    TitleButton *tb = malloc (sizeof(TitleButton));
 
     if (!tb) {
 	fprintf (stderr,
@@ -241,7 +252,7 @@ int CreateTitleButton (char *name, int func, char *action, MenuRoot *menuroot,
 
     /*
      * Cases for list:
-     * 
+     *
      *     1.  empty list, prepend left       put at head of list
      *     2.  append left, prepend right     put in between left and right
      *     3.  append right                   put at tail of list
@@ -278,11 +289,11 @@ int CreateTitleButton (char *name, int func, char *action, MenuRoot *menuroot,
 
 
 /**
- * Do all the necessary stuff to load in a titlebar button.  If we can't find 
- * the button, then put in a question; if we can't find the question mark, 
+ * Do all the necessary stuff to load in a titlebar button.  If we can't find
+ * the button, then put in a question; if we can't find the question mark,
  * something is wrong and we are probably going to be in trouble later on.
  */
-void InitTitlebarButtons ()
+void InitTitlebarButtons (void)
 {
     TitleButton *tb;
     int h;
@@ -398,7 +409,7 @@ PaintEntry(MenuRoot *mr, MenuItem *mi, int exposure)
 	    else
 		gc = Scr->MenuGC;
 
-	    MyFont_DrawString(dpy, mr->w, &Scr->MenuFont, gc, 
+	    MyFont_DrawString(dpy, mr->w, &Scr->MenuFont, gc,
 		    mi->x, text_y, mi->item, mi->strlen);
 
 	}
@@ -443,7 +454,7 @@ PaintEntry(MenuRoot *mr, MenuItem *mi, int exposure)
 	    text_y, mi->item, mi->strlen);
     }
 }
-    
+
 
 void
 PaintMenu(MenuRoot *mr, XEvent *e)
@@ -471,12 +482,13 @@ PaintMenu(MenuRoot *mr, XEvent *e)
 static Bool fromMenu;
 
 void
-UpdateMenu()
+UpdateMenu(void)
 {
     MenuItem *mi;
     int i, x, y, x_root, y_root, entry;
     int done;
     MenuItem *badItem = NULL;
+    XPointer context_data;
 
     fromMenu = TRUE;
 
@@ -510,15 +522,19 @@ UpdateMenu()
 	if (Event.type != MotionNotify)
 	    continue;
 
+	if (!ActiveMenu)
+            continue;
+
 	done = FALSE;
 	XQueryPointer( dpy, ActiveMenu->w, &JunkRoot, &JunkChild,
 	    &x_root, &y_root, &x, &y, &JunkMask);
 
 	/* if we haven't recieved the enter notify yet, wait */
-	if (ActiveMenu && !ActiveMenu->entered)
+	if (!ActiveMenu->entered)
 	    continue;
 
-	XFindContext(dpy, ActiveMenu->w, ScreenContext, (caddr_t *)&Scr);
+	if (XFindContext(dpy, ActiveMenu->w, ScreenContext, &context_data) == 0)
+	    Scr = (struct ScreenInfo *) context_data;
 
 	if (x < 0 || y < 0 ||
 	    x >= ActiveMenu->width || y >= ActiveMenu->height)
@@ -548,7 +564,7 @@ UpdateMenu()
 		done = TRUE;
 
 	    /* if we weren't on the active entry, let's turn the old
-	     * active one off 
+	     * active one off
 	     */
 	    if (!done && ActiveItem->func != F_TITLE)
 	    {
@@ -558,12 +574,12 @@ UpdateMenu()
 	}
 
 	/* if we weren't on the active item, change the active item and turn
-	 * it on 
+	 * it on
 	 */
 	if (!done)
 	{
 	    ActiveItem = mi;
-	    if (ActiveItem->func != F_TITLE && !ActiveItem->state)
+	    if (ActiveItem && ActiveItem->func != F_TITLE && !ActiveItem->state)
 	    {
 		ActiveItem->state = 1;
 		PaintEntry(ActiveMenu, ActiveItem, False);
@@ -571,16 +587,16 @@ UpdateMenu()
 	}
 
 	/* now check to see if we were over the arrow of a pull right entry */
-	if (ActiveItem->func == F_MENU && 
+	if (ActiveItem && ActiveItem->func == F_MENU &&
 	    ((ActiveMenu->width - x) < (ActiveMenu->width >> 1)))
 	{
 	    MenuRoot *save = ActiveMenu;
-	    int savex = MenuOrigins[MenuDepth - 1].x; 
+	    int savex = MenuOrigins[MenuDepth - 1].x;
 	    int savey = MenuOrigins[MenuDepth - 1].y;
 
 	    if (MenuDepth < MAXMENUDEPTH) {
-		PopUpMenu (ActiveItem->sub, 
-			   (savex + (ActiveMenu->width >> 1)), 
+		PopUpMenu (ActiveItem->sub,
+			   (savex + (ActiveMenu->width >> 1)),
 			   (savey + ActiveItem->item_num * Scr->EntryHeight)
 			   /*(savey + ActiveItem->item_num * Scr->EntryHeight +
 			    (Scr->EntryHeight >> 1))*/, False);
@@ -611,13 +627,13 @@ UpdateMenu()
  *  \param name  the name of the menu root
  */
 MenuRoot *
-NewMenuRoot(char *name)
+NewMenuRoot(const char *name)
 {
     MenuRoot *tmp;
 
 #define UNUSED_PIXEL ((unsigned long) (~0))	/* more than 24 bits */
 
-    tmp = (MenuRoot *) malloc(sizeof(MenuRoot));
+    tmp = malloc(sizeof(MenuRoot));
     tmp->hi_fore = UNUSED_PIXEL;
     tmp->hi_back = UNUSED_PIXEL;
     tmp->name = name;
@@ -670,8 +686,8 @@ NewMenuRoot(char *name)
  *  \param back   background color string
  */
 MenuItem *
-AddToMenu(MenuRoot *menu, char *item, char *action, MenuRoot *sub, int func, 
-          char *fore, char *back)
+AddToMenu(MenuRoot *menu, const char *item, const char *action,
+          MenuRoot *sub, int func, const char *fore, const char *back)
 {
     MenuItem *tmp;
     int width;
@@ -681,7 +697,7 @@ AddToMenu(MenuRoot *menu, char *item, char *action, MenuRoot *sub, int func,
 	item, action, sub, func);
 #endif
 
-    tmp = (MenuItem *) malloc(sizeof(MenuItem));
+    tmp = malloc(sizeof(MenuItem));
     tmp->root = menu;
 
     if (menu->first == NULL)
@@ -735,7 +751,7 @@ AddToMenu(MenuRoot *menu, char *item, char *action, MenuRoot *sub, int func,
 
 
 void
-MakeMenus()
+MakeMenus(void)
 {
     MenuRoot *mr;
 
@@ -749,7 +765,7 @@ MakeMenus()
 }
 
 
-void
+static void
 MakeMenu(MenuRoot *mr)
 {
     MenuItem *start, *end, *cur, *tmp;
@@ -804,10 +820,10 @@ MakeMenu(MenuRoot *mr)
 		attributes.save_under = True;
 	    }
 	    mr->shadow = XCreateWindow (dpy, Scr->Root, 0, 0,
-					(unsigned int) mr->width, 
+					(unsigned int) mr->width,
 					(unsigned int) mr->height,
 					(unsigned int)0,
-					CopyFromParent, 
+					CopyFromParent,
 					(unsigned int) CopyFromParent,
 					(Visual *) CopyFromParent,
 					valuemask, &attributes);
@@ -914,7 +930,7 @@ MakeMenu(MenuRoot *mr)
 	b3.flags = DoRed | DoGreen | DoBlue;
 
 	num -= 1;
-	for (i = 0, cur = start->next; i < num; i++, cur = cur->next)
+	for (i = 0, cur = start->next; i < num && cur; i++, cur = cur->next)
 	{
 	    f3.red += fred;
 	    f3.green += fgreen;
@@ -948,14 +964,14 @@ MakeMenu(MenuRoot *mr)
  *  \param x,y    location of upper left of menu
  *  \param center whether or not to center horizontally over position
  */
-Bool 
+Bool
 PopUpMenu (MenuRoot *menu, int x, int y, Bool center)
 {
     int WindowNameCount;
     TwmWindow **WindowNames;
     TwmWindow *tmp_win2,*tmp_win3;
     int i;
-    int (*compar)(const char *, const char *) = 
+    int (*compar)(const char *, const char *) =
       (Scr->CaseSensitive ? strcmp : XmuCompareISOLatin1);
 
     if (!menu) return False;
@@ -976,15 +992,14 @@ PopUpMenu (MenuRoot *menu, int x, int y, Bool center)
 	menu->width = 0;
 	menu->mapped = NEVER_MAPPED;
   	AddToMenu(menu, "TWM Windows", NULLSTR, NULL, F_TITLE,NULLSTR,NULLSTR);
-  
+
         for(tmp_win = Scr->TwmRoot.next , WindowNameCount=0;
             tmp_win != NULL;
             tmp_win = tmp_win->next)
           WindowNameCount++;
         if (WindowNameCount != 0)
         {
-            WindowNames =
-              (TwmWindow **)malloc(sizeof(TwmWindow *)*WindowNameCount);
+            WindowNames = malloc(sizeof(TwmWindow *) * WindowNameCount);
             WindowNames[0] = Scr->TwmRoot.next;
             for(tmp_win = Scr->TwmRoot.next->next , WindowNameCount=1;
                 tmp_win != NULL;
@@ -1076,7 +1091,7 @@ PopUpMenu (MenuRoot *menu, int x, int y, Bool center)
  * unhighlight the current menu selection and take down the menus
  */
 void
-PopDownMenu()
+PopDownMenu(void)
 {
     MenuRoot *tmp;
 
@@ -1117,7 +1132,7 @@ PopDownMenu()
  *  \param name the name of the menu root
  */
 MenuRoot *
-FindMenuRoot(char *name)
+FindMenuRoot(const char *name)
 {
     MenuRoot *tmp;
 
@@ -1131,14 +1146,14 @@ FindMenuRoot(char *name)
 
 
 
-static Bool 
+static Bool
 belongs_to_twm_window (TwmWindow *t, Window w)
 {
     if (!t) return False;
 
     if (w == t->frame || w == t->title_w || w == t->hilite_w ||
 	w == t->icon_w || w == t->icon_bm_w) return True;
-    
+
     if (t && t->titlebuttons) {
 	register TBWindow *tbw;
 	register int nb = Scr->TBInfo.nleft + Scr->TBInfo.nright;
@@ -1151,7 +1166,7 @@ belongs_to_twm_window (TwmWindow *t, Window w)
 
 
 
-void 
+static void
 resizeFromCenter(Window w, TwmWindow *tmp_win)
 {
   int lastx, lasty, bw2;
@@ -1171,11 +1186,11 @@ resizeFromCenter(Window w, TwmWindow *tmp_win)
   height = Scr->SizeFont.height + SIZE_VINDENT * 2;
 #endif
   XGetGeometry(dpy, w, &JunkRoot, &origDragX, &origDragY,
-	       (unsigned int *)&DragWidth, (unsigned int *)&DragHeight, 
+	       (unsigned int *)&DragWidth, (unsigned int *)&DragHeight,
 	       &JunkBW, &JunkDepth);
   XWarpPointer(dpy, None, w,
-	       0, 0, 0, 0, DragWidth/2, DragHeight/2);   
-  XQueryPointer (dpy, Scr->Root, &JunkRoot, 
+	       0, 0, 0, 0, DragWidth/2, DragHeight/2);
+  XQueryPointer (dpy, Scr->Root, &JunkRoot,
 		 &JunkChild, &JunkX, &JunkY,
 		 &AddingX, &AddingY, &JunkMask);
 #if 0
@@ -1201,7 +1216,7 @@ resizeFromCenter(Window w, TwmWindow *tmp_win)
     {
       XMaskEvent(dpy,
 		 ButtonPressMask | PointerMotionMask, &event);
-      
+
       if (event.type == MotionNotify) {
 	/* discard any extra motion events before a release */
 	while(XCheckMaskEvent(dpy,
@@ -1209,38 +1224,38 @@ resizeFromCenter(Window w, TwmWindow *tmp_win)
 	  if (event.type == ButtonPress)
 	    break;
       }
-      
+
       if (event.type == ButtonPress)
 	{
 	  MenuEndResize(tmp_win);
 	  XMoveResizeWindow(dpy, w, AddingX, AddingY, AddingW, AddingH);
 	  break;
 	}
-      
+
 /*    if (!DispatchEvent ()) continue; */
 
       if (event.type != MotionNotify) {
 	continue;
       }
-      
+
       /*
        * XXX - if we are going to do a loop, we ought to consider
-       * using multiple GXxor lines so that we don't need to 
+       * using multiple GXxor lines so that we don't need to
        * grab the server.
        */
       XQueryPointer(dpy, Scr->Root, &JunkRoot, &JunkChild,
 		    &JunkX, &JunkY, &AddingX, &AddingY, &JunkMask);
-      
+
       if (lastx != AddingX || lasty != AddingY)
 	{
 	  MenuDoResize(AddingX, AddingY, tmp_win);
-	  
+
 	  lastx = AddingX;
 	  lasty = AddingY;
 	}
-      
+
     }
-} 
+}
 
 
 
@@ -1248,32 +1263,32 @@ resizeFromCenter(Window w, TwmWindow *tmp_win)
  * execute a twm root function.
  *
  *  \param func     the function to execute
- *  \param action   the menu action to execute 
+ *  \param action   the menu action to execute
  *  \param w        the window to execute this function on
  *  \param tmp_win  the twm window structure
  *  \param event    the event that caused the function
  *  \param context  the context in which the button was pressed
  *  \param pulldown flag indicating execution from pull down menu
  *
- *  \return TRUE if should continue with remaining actions, 
+ *  \return TRUE if should continue with remaining actions,
  *           else FALSE to abort
  */
 
-int
-WarpThere(TwmWindow *t) 
+static int
+WarpThere(TwmWindow *t)
 {
     if (Scr->WarpUnmapped || t->mapped) {
         if (!t->mapped) DeIconify (t);
         if (!Scr->NoRaiseWarp) XRaiseWindow (dpy, t->frame);
-        WarpToWindow (t); 
-        return 1; 
-    }    
+        WarpToWindow (t);
+        return 1;
+    }
     return 0;
 }
 
 
 int
-ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win, 
+ExecuteFunction(int func, const char *action, Window w, TwmWindow *tmp_win,
                 XEvent *eventp, int context, int pulldown)
 {
     static Time last_time = 0;
@@ -1384,7 +1399,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 		SortIconManager((IconMgr *) NULL);
 	    else if (tmp_win->iconmgr)
 		SortIconManager(tmp_win->iconmgrp);
-	    else 
+	    else
 		Bell(XkbBI_Info,0,tmp_win->w);
 
 	    Scr->SortIconMgr = save_sort;
@@ -1439,39 +1454,39 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 	PopDownMenu();
 
 	if (pulldown)
-	    XWarpPointer(dpy, None, Scr->Root, 
+	    XWarpPointer(dpy, None, Scr->Root,
 		0, 0, 0, 0, eventp->xbutton.x_root, eventp->xbutton.y_root);
 
 	if (w != tmp_win->icon_w) {	/* can't resize icons */
 
 	  if ((Context == C_FRAME || Context == C_WINDOW || Context == C_TITLE)
-	      && fromMenu) 
+	      && fromMenu)
 	    resizeFromCenter(w, tmp_win);
 	  else {
 	    /*
 	     * see if this is being done from the titlebar
 	     */
-	    fromtitlebar = 
+	    fromtitlebar =
 	      belongs_to_twm_window (tmp_win, eventp->xbutton.window);
-	    
+
 	    /* Save pointer position so we can tell if it was moved or
 	       not during the resize. */
 	    ResizeOrigX = eventp->xbutton.x_root;
 	    ResizeOrigY = eventp->xbutton.y_root;
-	    
+
 	    StartResize (eventp, tmp_win, fromtitlebar);
-	    
+
 	    do {
 	      XMaskEvent(dpy,
 			   ButtonPressMask | ButtonReleaseMask |
 			   EnterWindowMask | LeaveWindowMask |
 			   ButtonMotionMask, &Event);
-		
+
 		if (fromtitlebar && Event.type == ButtonPress) {
 		  fromtitlebar = False;
 		    continue;
 		  }
-		
+
 	    	if (Event.type == MotionNotify) {
 		  /* discard any extra motion events before a release */
 		  while
@@ -1480,13 +1495,13 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 		      if (Event.type == ButtonRelease)
 			break;
 		}
-	      
+
 	      if (!DispatchEvent ()) continue;
-	      
+
 	    } while (!(Event.type == ButtonRelease || Cancel));
 	    return TRUE;
 	  }
-	} 
+	}
 	break;
 
 
@@ -1512,7 +1527,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 	MoveFunction = func;
 
 	if (pulldown)
-	    XWarpPointer(dpy, None, Scr->Root, 
+	    XWarpPointer(dpy, None, Scr->Root,
 		0, 0, 0, 0, eventp->xbutton.x_root, eventp->xbutton.y_root);
 
 	EventHandler[EnterNotify] = HandleUnknown;
@@ -1538,8 +1553,8 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 	else if (w != tmp_win->icon_w)
 	{
 	    XTranslateCoordinates(dpy, w, tmp_win->frame,
-		eventp->xbutton.x, 
-		eventp->xbutton.y, 
+		eventp->xbutton.x,
+		eventp->xbutton.y,
 		&DragX, &DragY, &JunkChild);
 
 	    w = tmp_win->frame;
@@ -1560,7 +1575,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 	 * only do the constrained move if timer is set; need to check it
 	 * in case of stupid or wicked fast servers
 	 */
-	if (ConstrainedMoveTime && 
+	if (ConstrainedMoveTime &&
 	    (eventp->xbutton.time - last_time) < ConstrainedMoveTime)
 	{
 	    int width, height;
@@ -1619,15 +1634,15 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 
 	if (menuFromFrameOrWindowOrTitlebar) {
 	  /* warp the pointer to the middle of the window */
-	  XWarpPointer(dpy, None, Scr->Root, 0, 0, 0, 0, 
-		       origDragX + DragWidth / 2, 
+	  XWarpPointer(dpy, None, Scr->Root, 0, 0, 0, 0,
+		       origDragX + DragWidth / 2,
 		       origDragY + DragHeight / 2);
 	  XFlush(dpy);
 	}
-	
+
 	while (TRUE)
 	{
-	    long releaseEvent = menuFromFrameOrWindowOrTitlebar ? 
+	    long releaseEvent = menuFromFrameOrWindowOrTitlebar ?
 	                          ButtonPress : ButtonRelease;
 	    long movementMask = menuFromFrameOrWindowOrTitlebar ?
 	                          PointerMotionMask : ButtonMotionMask;
@@ -1640,7 +1655,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 
 	    /* throw away enter and leave events until release */
 	    if (Event.xany.type == EnterNotify ||
-		Event.xany.type == LeaveNotify) continue; 
+		Event.xany.type == LeaveNotify) continue;
 
 	    if (Event.type == MotionNotify) {
 		/* discard any extra motion events before a logical release */
@@ -1687,7 +1702,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 		      CurrentDragY != origDragY)))
 		  tmp_win->icon_moved = TRUE;
 		if (!Scr->OpaqueMove && menuFromFrameOrWindowOrTitlebar)
-		  XMoveWindow(dpy, DragWindow, 
+		  XMoveWindow(dpy, DragWindow,
 			      Event.xbutton.x_root - DragWidth / 2,
 			      Event.xbutton.y_root - DragHeight / 2);
 		break;
@@ -1768,7 +1783,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 			XMoveWindow(dpy, DragWindow, xl, yt);
 		    else
 			MoveOutline(eventp->xmotion.root, xl, yt, w, h,
-			    tmp_win->frame_bw, 
+			    tmp_win->frame_bw,
 			    moving_icon ? 0 : tmp_win->title_height);
 		}
 	    }
@@ -1782,7 +1797,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 		else {
 		  xl = eventp->xmotion.x_root - (DragWidth / 2);
 		  yt = eventp->xmotion.y_root - (DragHeight / 2);
-		}		  
+		}
 		w = DragWidth + 2 * JunkBW;
 		h = DragHeight + 2 * JunkBW;
 
@@ -1828,7 +1843,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 
 	    if ((mroot = FindMenuRoot(action)) == NULL)
 	    {
-		fprintf (stderr, "%s: couldn't find function \"%s\"\n", 
+		fprintf (stderr, "%s: couldn't find function \"%s\"\n",
 			 ProgramName, action);
 		return TRUE;
 	    }
@@ -1876,13 +1891,13 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 	    XConfigureWindow (dpy, w, CWStackMode, &xwc);
 	}
 	break;
-	
+
     case F_RAISE:
 	if (DeferExecution(context, func, Scr->SelectCursor))
 	    return TRUE;
 
 	/* check to make sure raise is not from the WindowFunction */
-	if (w == tmp_win->icon_w && Context != C_ROOT) 
+	if (w == tmp_win->icon_w && Context != C_ROOT)
 	    XRaiseWindow(dpy, tmp_win->icon_w);
 	else
 	    XRaiseWindow(dpy, tmp_win->frame);
@@ -2001,12 +2016,12 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 			if (count > 0) XStoreBytes (dpy, buff, count);
 			close(fd);
 		    } else {
-			fprintf (stderr, 
-				 "%s:  unable to open cut file \"%s\"\n", 
+			fprintf (stderr,
+				 "%s:  unable to open cut file \"%s\"\n",
 				 ProgramName, tmp);
 		    }
 		    if (ptr != tmp) free (ptr);
-		} 
+		}
 	    } else {
 		XFree(ptr);
 	    }
@@ -2083,23 +2098,23 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 	    len = strlen(action);
 
 	    for (t = Scr->TwmRoot.next; t != NULL; t = t->next) {
-		if (!strncmp(action, t->name, len)) 
+		if (!strncmp(action, t->name, len))
                     if (WarpThere(t)) break;
 	    }
 	    if (!t) {
 		for (t = Scr->TwmRoot.next; t != NULL; t = t->next) {
-		    if (!strncmp(action, t->class.res_name, len)) 
+		    if (!strncmp(action, t->class.res_name, len))
                         if (WarpThere(t)) break;
 		}
 		if (!t) {
 		    for (t = Scr->TwmRoot.next; t != NULL; t = t->next) {
-			if (!strncmp(action, t->class.res_class, len)) 
+			if (!strncmp(action, t->class.res_class, len))
                             if (WarpThere(t)) break;
 		    }
 		}
 	    }
 
-	    if (!t) 
+	    if (!t)
 		Bell(XkbBI_MinorError,0,None);
 	}
 	break;
@@ -2139,7 +2154,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 	    }
 	}
 	break;
-	
+
     case F_WARPRING:
 	switch (action[0]) {
 	  case 'n':
@@ -2167,7 +2182,7 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 	}
 	else
 	{
-	    fprintf (stderr, "%s:  unable to open file \"%s\"\n", 
+	    fprintf (stderr, "%s:  unable to open file \"%s\"\n",
 		     ProgramName, ptr);
 	}
 	if (ptr != action) free(ptr);
@@ -2237,14 +2252,14 @@ ExecuteFunction(int func, char *action, Window w, TwmWindow *tmp_win,
 
 
 /**
- * defer the execution of a function to the next button press if the context 
+ * defer the execution of a function to the next button press if the context
  *  is C_ROOT
  *
  *  \param context the context in which the mouse button was pressed
  *	\param func    the function to defer
  *  \param cursor  cursor the cursor to display while waiting
  */
-int
+static int
 DeferExecution(int context, int func, Cursor cursor)
 {
   if (context == C_ROOT)
@@ -2259,7 +2274,7 @@ DeferExecution(int context, int func, Cursor cursor)
 
 	return (TRUE);
     }
-    
+
     return (FALSE);
 }
 
@@ -2269,7 +2284,7 @@ DeferExecution(int context, int func, Cursor cursor)
  *regrab the pointer with the LastCursor;
  */
 void
-ReGrab()
+ReGrab(void)
 {
     XGrabPointer(dpy, Scr->Root, True,
 	ButtonPressMask | ButtonReleaseMask,
@@ -2280,12 +2295,12 @@ ReGrab()
 
 
 /**
- * checks each function in the list to see if it is one that needs 
+ * checks each function in the list to see if it is one that needs
  * to be deferred.
  *
  *  \param root the menu root to check
  */
-Bool
+static Bool
 NeedToDefer(MenuRoot *root)
 {
     MenuItem *mitem;
@@ -2329,13 +2344,13 @@ NeedToDefer(MenuRoot *root)
  * execute the string by /bin/sh
  *  \param s  the string containing the command
  */
-static int 
-System (char *s)
+static int
+System (const char *s)
 {
     int pid, status;
     if ((pid = fork ()) == 0) {
 	(void) setpgrp();
-	execl ("/bin/sh", "sh", "-c", s, 0);
+	execl ("/bin/sh", "sh", "-c", s, NULL);
     } else
 	waitpid (pid, &status, 0);
     return status;
@@ -2344,8 +2359,8 @@ System (char *s)
 
 #endif
 
-void
-Execute(char *s)
+static void
+Execute(const char *s)
 {
 	/* FIXME: is all this stuff needed?  There could be security problems here. */
     static char buf[256];
@@ -2381,7 +2396,7 @@ Execute(char *s)
     (void) system (s);
 
     if (restorevar) {		/* why bother? */
-	(void) sprintf (buf, "DISPLAY=%s", oldDisplay);
+	(void) snprintf (buf, sizeof(buf), "DISPLAY=%s", oldDisplay);
 	putenv (buf);
     }
 }
@@ -2392,7 +2407,7 @@ Execute(char *s)
  * put input focus on the root window.
  */
 void
-FocusOnRoot()
+FocusOnRoot(void)
 {
     SetFocus ((TwmWindow *) NULL, LastTimestamp());
     if (Scr->Focus != NULL)
@@ -2459,7 +2474,7 @@ DeIconify(TwmWindow *tmp_win)
 		Zoom(t->icon_w, t->frame);
 	      else
 		Zoom(tmp_win->icon_w, t->frame);
-	      
+
 	      XMapWindow(dpy, t->w);
 	      t->mapped = TRUE;
 	      if (Scr->NoRaiseDeicon)
@@ -2467,7 +2482,7 @@ DeIconify(TwmWindow *tmp_win)
 	      else
 		XMapRaised(dpy, t->frame);
 	      SetMapStateProp(t, NormalState);
-	      
+
 	      if (t->icon_w) {
 		XUnmapWindow(dpy, t->icon_w);
 		IconDown (t);
@@ -2477,7 +2492,7 @@ DeIconify(TwmWindow *tmp_win)
 	      t->icon_on = FALSE;
 	    }
 	}
-    
+
     XSync (dpy, 0);
 }
 
@@ -2518,7 +2533,7 @@ Iconify(TwmWindow *tmp_win, int def_x, int def_y)
 		else
 		  Zoom(t->frame, tmp_win->icon_w);
 	      }
-	    
+
 	    /*
 	     * Prevent the receipt of an UnmapNotify, since that would
 	     * cause a transition to the Withdrawn state.
@@ -2542,8 +2557,8 @@ Iconify(TwmWindow *tmp_win, int def_x, int def_y)
 	    t->icon = TRUE;
 	    t->icon_on = FALSE;
 	  }
-      } 
-    
+      }
+
     if (iconify)
 	Zoom(tmp_win->frame, tmp_win->icon_w);
 
@@ -2575,7 +2590,7 @@ Iconify(TwmWindow *tmp_win, int def_x, int def_y)
 
 
 
-static void 
+static void
 Identify (TwmWindow *t)
 {
     int i, n, twidth, width, height;
@@ -2586,7 +2601,7 @@ Identify (TwmWindow *t)
     unsigned udummy;
 
     n = 0;
-    (void) sprintf(Info[n++], "Twm version:  %s", Version);
+    snprintf(Info[n++], INFO_SIZE, "Twm version:  %s", Version);
     Info[n++][0] = '\0';
 
     if (t) {
@@ -2594,31 +2609,34 @@ Identify (TwmWindow *t)
 		      &wwidth, &wheight, &bw, &depth);
 	(void) XTranslateCoordinates (dpy, t->w, Scr->Root, 0, 0,
 				      &x, &y, &junk);
-	(void) sprintf(Info[n++], "Name             = \"%s\"", t->full_name);
-	(void) sprintf(Info[n++], "Class.res_name   = \"%s\"", t->class.res_name);
-	(void) sprintf(Info[n++], "Class.res_class  = \"%s\"", t->class.res_class);
+	snprintf(Info[n++], INFO_SIZE,
+		 "Name             = \"%s\"", t->full_name);
+	snprintf(Info[n++], INFO_SIZE,
+		 "Class.res_name   = \"%s\"", t->class.res_name);
+	snprintf(Info[n++], INFO_SIZE,
+		 "Class.res_class  = \"%s\"", t->class.res_class);
 	Info[n++][0] = '\0';
-	(void) sprintf(Info[n++], "Geometry/root    = %dx%d+%d+%d", wwidth, wheight,
-		x, y);
-	(void) sprintf(Info[n++], "Border width     = %d", bw);
-	(void) sprintf(Info[n++], "Depth            = %d", depth);
+	snprintf(Info[n++], INFO_SIZE,
+		 "Geometry/root    = %dx%d+%d+%d", wwidth, wheight, x, y);
+	snprintf(Info[n++], INFO_SIZE, "Border width     = %d", bw);
+	snprintf(Info[n++], INFO_SIZE, "Depth            = %d", depth);
 	if (HasSync)
 	{
 	    int priority;
 	    (void)XSyncGetPriority(dpy, t->w, &priority);
-	    (void) sprintf(Info[n++], "Priority         = %d", priority);
+	    snprintf(Info[n++], INFO_SIZE, "Priority         = %d", priority);
 	}
     }
 
     Info[n++][0] = '\0';
-    (void) sprintf(Info[n++], "Click to dismiss....");
+    snprintf(Info[n++], INFO_SIZE, "Click to dismiss....");
 
     /* figure out the width and height of the info window */
     height = n * (Scr->DefaultFont.height+2);
     width = 1;
     for (i = 0; i < n; i++)
     {
-	twidth = MyFont_TextWidth(&Scr->DefaultFont, Info[i], 
+	twidth = MyFont_TextWidth(&Scr->DefaultFont, Info[i],
 	    strlen(Info[i]));
 	if (twidth > width)
 	    width = twidth;
@@ -2630,9 +2648,9 @@ Identify (TwmWindow *t)
 		       &dummy, &dummy, &udummy)) {
 	px -= (width / 2);
 	py -= (height / 3);
-	if (px + width + BW2 >= Scr->MyDisplayWidth) 
+	if (px + width + BW2 >= Scr->MyDisplayWidth)
 	  px = Scr->MyDisplayWidth - width - BW2;
-	if (py + height + BW2 >= Scr->MyDisplayHeight) 
+	if (py + height + BW2 >= Scr->MyDisplayHeight)
 	  py = Scr->MyDisplayHeight - height - BW2;
 	if (px < 0) px = 0;
 	if (py < 0) py = 0;
@@ -2640,7 +2658,7 @@ Identify (TwmWindow *t)
 	px = py = 0;
     }
     XMoveResizeWindow(dpy, Scr->InfoWindow, px, py, width, height);
-    XMapRaised(dpy, Scr->InfoWindow); 
+    XMapRaised(dpy, Scr->InfoWindow);
     InfoLines = n;
 }
 
@@ -2649,38 +2667,39 @@ void
 SetMapStateProp(TwmWindow *tmp_win, int state)
 {
     unsigned long data[2];		/* "suggested" by ICCCM version 1 */
-  
+
     data[0] = (unsigned long) state;
-    data[1] = (unsigned long) (tmp_win->iconify_by_unmapping ? None : 
+    data[1] = (unsigned long) (tmp_win->iconify_by_unmapping ? None :
 			   tmp_win->icon_w);
 
-    XChangeProperty (dpy, tmp_win->w, _XA_WM_STATE, _XA_WM_STATE, 32, 
+    XChangeProperty (dpy, tmp_win->w, _XA_WM_STATE, _XA_WM_STATE, 32,
 		 PropModeReplace, (unsigned char *) data, 2);
 }
 
 
 
-Bool 
+Bool
 GetWMState (Window w, int *statep, Window *iwp)
 {
     Atom actual_type;
     int actual_format;
     unsigned long nitems, bytesafter;
-    unsigned long *datap = NULL;
+    unsigned char *prop_return = NULL;
     Bool retval = False;
 
     if (XGetWindowProperty (dpy, w, _XA_WM_STATE, 0L, 2L, False, _XA_WM_STATE,
 			    &actual_type, &actual_format, &nitems, &bytesafter,
-			    (unsigned char **) &datap) != Success || !datap)
+			    &prop_return) != Success || !prop_return)
       return False;
 
     if (nitems <= 2) {			/* "suggested" by ICCCM version 1 */
+	unsigned long *datap = (unsigned long *) prop_return;
 	*statep = (int) datap[0];
 	*iwp = (Window) datap[1];
 	retval = True;
     }
 
-    XFree ((char *) datap);
+    XFree (prop_return);
     return retval;
 }
 
@@ -2695,7 +2714,7 @@ WarpToScreen (int n, int inc)
 
     while (!newscr) {
 					/* wrap around */
-	if (n < 0) 
+	if (n < 0)
 	  n = NumScreens - 1;
 	else if (n >= NumScreens)
 	  n = 0;
@@ -2706,7 +2725,7 @@ WarpToScreen (int n, int inc)
 		n += inc;
 		continue;
 	    }
-	    fprintf (stderr, "%s:  unable to warp to unmanaged screen %d\n", 
+	    fprintf (stderr, "%s:  unable to warp to unmanaged screen %d\n",
 		     ProgramName, n);
 	    Bell(XkbBI_MinorError,0,None);
 	    return;
@@ -2729,7 +2748,7 @@ WarpToScreen (int n, int inc)
 /**
  * rotate our internal copy of WM_COLORMAP_WINDOWS
  */
-void
+static void
 BumpWindowColormap (TwmWindow *tmp, int inc)
 {
     int i, j, previously_installed;
@@ -2738,9 +2757,8 @@ BumpWindowColormap (TwmWindow *tmp, int inc)
     if (!tmp) return;
 
     if (inc && tmp->cmaps.number_cwins > 0) {
-	cwins = (ColormapWindow **) malloc(sizeof(ColormapWindow *)*
-					   tmp->cmaps.number_cwins);
-	if (cwins) {		
+	cwins = malloc(sizeof(ColormapWindow *) * tmp->cmaps.number_cwins);
+	if (cwins) {
 	    if ((previously_installed =
 		/* SUPPRESS 560 */(Scr->cmapInfo.cmaps == &tmp->cmaps &&
 	        tmp->cmaps.number_cwins))) {
@@ -2757,12 +2775,12 @@ BumpWindowColormap (TwmWindow *tmp, int inc)
 		cwins[j] = tmp->cmaps.cwins[i];
 	    }
 
-	    free((char *) tmp->cmaps.cwins);
+	    free(tmp->cmaps.cwins);
 
 	    tmp->cmaps.cwins = cwins;
 
 	    if (tmp->cmaps.number_cwins > 1)
-		bzero (tmp->cmaps.scoreboard, 
+		bzero (tmp->cmaps.scoreboard,
 		       ColormapsScoreboardLength(&tmp->cmaps));
 
 	    if (previously_installed)
@@ -2773,8 +2791,8 @@ BumpWindowColormap (TwmWindow *tmp, int inc)
 }
 
 
-void
-HideIconManager ()
+static void
+HideIconManager (void)
 {
     SetMapStateProp (Scr->iconmgr.twm_win, WithdrawnState);
     XUnmapWindow(dpy, Scr->iconmgr.twm_win->frame);
@@ -2792,18 +2810,18 @@ SetBorder (TwmWindow *tmp, Bool onoroff)
     if (tmp->highlight) {
 	if (onoroff) {
 	    XSetWindowBorder (dpy, tmp->frame, tmp->border);
-	    if (tmp->title_w) 
+	    if (tmp->title_w)
 	      XSetWindowBorder (dpy, tmp->title_w, tmp->border);
 	} else {
 	    XSetWindowBorderPixmap (dpy, tmp->frame, tmp->gray);
-	    if (tmp->title_w) 
+	    if (tmp->title_w)
 	      XSetWindowBorderPixmap (dpy, tmp->title_w, tmp->gray);
 	}
     }
 }
 
 
-void
+static void
 DestroyMenu (MenuRoot *menu)
 {
     MenuItem *item;
@@ -2818,7 +2836,7 @@ DestroyMenu (MenuRoot *menu)
     for (item = menu->first; item; ) {
 	MenuItem *tmp = item;
 	item = item->next;
-	free ((char *) tmp);
+	free (tmp);
     }
 }
 
@@ -2828,14 +2846,14 @@ DestroyMenu (MenuRoot *menu)
  * warping routines
  */
 
-void 
+static void
 WarpAlongRing (XButtonEvent *ev, Bool forward)
 {
     TwmWindow *r, *head;
 
     if (Scr->RingLeader)
       head = Scr->RingLeader;
-    else if (!(head = Scr->Ring)) 
+    else if (!(head = Scr->Ring))
       return;
 
     if (forward) {
@@ -2850,19 +2868,23 @@ WarpAlongRing (XButtonEvent *ev, Bool forward)
 
     if (r && r != head) {
 	TwmWindow *p = Scr->RingLeader, *t;
+	XPointer context_data;
 
 	Scr->RingLeader = r;
 	WarpToWindow (r);
 
-	if (p && p->mapped &&
-	    XFindContext (dpy, ev->window, TwmContext, (caddr_t *)&t) == XCSUCCESS &&
-	    p == t) {
+	if (XFindContext (dpy, ev->window, TwmContext, &context_data) == 0)
+	    t = (TwmWindow *) context_data;
+	else
+	    t = NULL;
+
+	if (p && p->mapped && p == t) {
 	    p->ring.cursor_valid = True;
 	    p->ring.curs_x = ev->x_root - t->frame_x;
 	    p->ring.curs_y = ev->y_root - t->frame_y;
-	    if (p->ring.curs_x < -p->frame_bw || 
+	    if (p->ring.curs_x < -p->frame_bw ||
 		p->ring.curs_x >= p->frame_width + p->frame_bw ||
-		p->ring.curs_y < -p->frame_bw || 
+		p->ring.curs_y < -p->frame_bw ||
 		p->ring.curs_y >= p->frame_height + p->frame_bw) {
 		/* somehow out of window */
 		p->ring.curs_x = p->frame_width / 2;
@@ -2874,7 +2896,7 @@ WarpAlongRing (XButtonEvent *ev, Bool forward)
 
 
 
-void 
+static void
 WarpToWindow (TwmWindow *t)
 {
     int x, y;
@@ -2904,7 +2926,7 @@ WarpToWindow (TwmWindow *t)
  *     data[0]		message atom
  *     data[1]		time stamp
  */
-static void 
+static void
 send_clientmessage (Window w, Atom a, Time timestamp)
 {
     XClientMessageEvent ev;
